import {Component, Input, OnInit} from '@angular/core';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {GameStatus} from '../interface/gameStatus';
import {ActivatedRoute, Router} from '@angular/router';
import {io} from 'socket.io-client';
import {environment} from '../../environments/environment';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';
@Component({
  selector: 'app-guesser',
  templateUrl: './guesser.component.html',
  styleUrls: ['./guesser.component.scss']
})
export class GuesserComponent implements OnInit {
  socket: any;
  sessionName = '';
  TIME_LIMIT = 100;
  timePassed = 0;
  timeRemaining = 60;
  timerInterval = 0;
  timeLeft = this.TIME_LIMIT;
  gameStatus: GameStatus = {
    sessionID: -1,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeWatchdog: 1
  };
  team = '';

  constructor(private router: Router,
              private activatedRoute: ActivatedRoute,
              private service: TabuMiddlewareService,
              private isAllowedToPlay: IsAllowedToPlay) { }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName'));
      this.team = String(params.get('team'));
      if (!this.isAllowedToPlay.isAllowed || this.isAllowedToPlay.role !== 'guesser') {
        this.router.navigate([this.sessionName + '/' + this.team]);
      }
      this.fillGamestatus();
      this.socket = io(environment.tabuServerURL);
      this.listen();
    });
    this.startTimer();
  }

  listen(): any {
    this.socket.on(this.sessionName + ':newCard', (data: any) => {
      this.service.getCard({cardID: data}).then(value => {
        this.fillGamestatus();
      });
    });
    this.socket.on(this.sessionName + ':endRound', (data: any) => {
      console.log('GAME: ', data);
      if (JSON.parse(data)) {
        console.log('GAME END: ', data);
        this.socket.disconnect();
        this.router.navigate([this.sessionName + '/' + this.team]);
      }
    });
  }

  onTimesUp(): void{
    this.socket.disconnect();
    this.router.navigate([this.sessionName + '/' + this.team]);
  }

  startTimer(): void {
    this.startCountdown();
    this.timerInterval = setInterval(() => {
      this.timePassed = this.timePassed += 1;
      this.timeLeft = (this.TIME_LIMIT - this.timePassed);
      if (this.timeLeft === 0) {
        console.log('Times up!');
        this.onTimesUp();
      }
    }, 593);
  }
  startCountdown(): void {
    setInterval(() => {
      this.timeRemaining = this.timeRemaining -= 1;
    }, 1000);
  }

  fillGamestatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.gameStatus.sessionID = JSON.parse(value.sessionID);
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
        this.gameStatus.redTurn = JSON.parse(value.redTurn);
        this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
        this.gameStatus.activeWatchdog = JSON.parse(value.activeWatchdog);
      }
    });
  }
}
